@@ -1,51 +1,51 @@
 package Hash::MD5;
 
+use utf8;
 use 5.006;
 use strict;
 use warnings FATAL => 'all';
 use Digest::MD5 qw(md5_hex);
-use vars qw($VERSION @ISA @EXPORT_OK);
-use Data::Dumper;
+use vars qw($VERSION @EXPORT_OK);
 
 require Exporter;
-*import = \&Exporter::import;
+*import    = \&Exporter::import;
 @EXPORT_OK = qw(sum_hash sum_array);
 
 ###############################################################################
 ##############                 PRIVATE SUBS              ######################
 ###############################################################################
-sub _sort_keys{
-	my ( $hashref ) = @_;
-	my $number_of_elements = scalar(keys %$hashref);
-	my $ret = "{";
+sub _sort_keys {
+	my ($hashref) = @_;
+	my $serial    = scalar( keys %$hashref );
+	my $ret       = '';
 	foreach ( sort { $a cmp $b } keys %$hashref ) {
-		if(ref $hashref->{$_} eq 'HASH'){
-			$ret = $ret.qq~$_:~._sort_keys($hashref->{$_});
-		}elsif(ref $hashref->{$_} eq 'ARRAY'){
-			$ret = $ret.qq~$_:~._sort_array($hashref->{$_});
-		}else{
-			$ret = $ret.qq~$_:"$hashref->{$_}"~;
+		if 		( ref $hashref->{$_} eq 'HASH' ) { $ret = $ret . unpack( 'H*', $_ ) . _sort_keys( ( defined $hashref->{$_} ? $hashref->{$_} : 'undef' ) ); }
+		elsif 	( ref $hashref->{$_} eq 'ARRAY' ){ $ret = $ret . unpack( 'H*', $_ ) . _sort_array(( defined $hashref->{$_} ? $hashref->{$_} : 'undef' ) ); }
+		else {
+			my $var = (defined $hashref->{$_}) ? $hashref->{$_} : 'undef';
+			$serial = $serial . length( $var );
+			utf8::encode($_);
+			utf8::encode($var);
+			$ret = $ret . unpack( 'H*', $_ ) . unpack( 'H*', $var );
 		}
-		$ret = $ret.",";
 	}
-	return $ret . "lenght:". $number_of_elements . "}";
+	return $ret . $serial;
 }
 
-sub _sort_array{
-	my ( $arrayref ) = @_;
-	my $number_of_elements = scalar( @$arrayref );
-	my $ret = "[";
-	foreach ( @$arrayref ) {
-		if(ref $_ eq 'HASH'){
-			$ret = $ret._sort_keys($_) ;
-		}elsif(ref $_ eq 'ARRAY'){
-			$ret = $ret._sort_array($_) ;
-		}else{
-			$ret = $ret.qq~"$_"~;	
+sub _sort_array {
+	my ($arrayref) = @_;
+	my $serial     = scalar(@$arrayref);
+	my $ret        = '';
+	foreach (@$arrayref) {
+		if 		( ref $_ eq 'HASH' ) { $ret = $ret . _sort_keys($_); }
+		elsif 	( ref $_ eq 'ARRAY') { $ret = $ret . _sort_array($_);}
+		else {
+			$serial = $serial . length($_);
+			utf8::encode($_);
+			$ret = $ret . unpack( 'H*', $_ );
 		}
-		$ret = $ret.",";
 	}
-	return $ret . $number_of_elements ."]";
+	return $ret . $serial;
 }
 
 =pod
@@ -56,12 +56,11 @@ Hash::MD5 - MD5 checksum for choosen hashref
 
 =head1 VERSION
 
-Version 0.04
+Version 0.05
 
 =cut
 
-our $VERSION = '0.04';
-
+our $VERSION = '0.05';
 
 =head1 SYNOPSIS
 
@@ -87,8 +86,8 @@ our $VERSION = '0.04';
 
 =cut
 
-sub sum_hash{
-	my ( $arg ) = @_;
+sub sum_hash {
+	my ($arg) = @_;
 	return md5_hex( _sort_keys($arg) );
 }
 
@@ -96,8 +95,8 @@ sub sum_hash{
 
 =cut
 
-sub sum_array{
-	my ( $arg ) = @_;
+sub sum_array {
+	my ($arg) = @_;
 	return md5_hex( _sort_array($arg) );
 }
 
@@ -135,7 +134,14 @@ L<http://search.cpan.org/dist/Hash-MD5/>
 =back
 
 
-=head1 ACKNOWLEDGEMENTS
+=head1 MOTIVATION
+
+I need a package which I can grab the uniqueness of a Hashrefs in a md5 sum.
+This I will compare to later processing.
+My first approach was to use encode_json string.
+Unfortunately, the resort was not very consistent.
+
+So I wrote my first cpan packet in order also to learn.
 
 
 =head1 LICENSE AND COPYRIGHT
@@ -181,4 +187,4 @@ EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
 =cut
 
-1; # End of Hash::MD5
\ No newline at end of file
+1;    # End of Hash::MD5
